	list		p=12F1822      ; list directive to define processor
	#include	<p12F1822.inc> ; processor specific variable definitions

    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_ON & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_ON & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_ON & _STVREN_OFF & _BORV_19 & _LVP_OFF

;-------------------------------------------------------------------------
;*****   
State		EQU 0x70			;   
								; b7 -   
								; b6 -     
								; b5 -    UART_Recv  
UART_WR		EQU	0x71			;       
UART_RD		EQU 0x72			;       
CMD_ADR		EQU 0x73			;    (0x73)
CMD_LEN		EQU 0x74			;   
CMD_TMR		EQU 0x75			;  2 byte (0x75, 0x76)
MQTT_Size	EQU 0x77			;  MQTT-
VAL_Buf		EQU 0x78			;   0x78, 0x79, 0x7A
DivVAL		EQU 0x7B			; 
RetVAL		EQU 0x7C			; 
TmpREG		EQU 0x7D			;   (0x7D, 0x7E)
RS485_LEN	EQU 0x7F			;     RS485
;**********************************************************************
UART_Buf	EQU 0x20			;  UART 80  (0x20 - 0x6F)
;*************************************************************************
RS485_Buf	EQU 0xA0			;     Bank 1 = 32 
;*************************************************************************
UART_1200	EQU D'6666'			; 0
UART_2400	EQU D'3332'			; 1
UART_4800	EQU D'1666'			; 2
UART_9600	EQU D'832'			; 3
UART_19200	EQU D'416'			; 4
UART_38400	EQU D'208'			; 5
UART_57600	EQU D'138'			; 6
UART_115200	EQU D'68'			; 7
;*************************************************************************
lw	MACRO	reg, value			;   2-  
	movlw	LOW value
	movwf	reg+0
	movlw	HIGH value
	movwf	reg+1
	ENDM

CMD_ASCII	MACRO val			;    UART
	movlw	val
	call	UART_SendASCII
			ENDM

CMD_Query	MACRO cmd			;    UART
	movlw	LOW cmd
	call	UART_Query
			ENDM

CMD_QueryCR	MACRO cmd			;    CR  UART
	movlw	LOW cmd
	call	UART_QueryCR
			ENDM

TMR_Set		MACRO val			;    val 
	movlw	val
	call	TMR_Set_
			ENDM

RPL_Wait	MACRO rpl			;    UART
	movlw	LOW rpl
	call	UART_Wait
			ENDM

RPL_Timeout	MACRO addr			;    
	btfsc	State, 5
	goto	addr
			ENDM

StateSet	MACRO val
	movlw	val
	call	StateSet_
			ENDM
;*************************************************************************
    ORG		0x0000            	; processor reset vector
    goto	MAIN_Start         	; When using debug header, first inst.
                              	; may be passed over by ICD2.  
;-------------------------------------------------------------------------
    ORG		0x0004
	
	movlb	0
	btfsc	PIR1, RCIF			;    UART
	call	RCEvent

	movlb	0					;  TMR2
	btfsc	PIR1, TMR2IF
	call	TMR2Event

    RETFIE                   	; return from interrupt

;=========================================================================
;    TMR2  0.01s
TMR2Event:
	bcf		PIR1, TMR2IF		;    TMR2

	movf	CMD_TMR+0, W
	iorwf	CMD_TMR+1, W
	btfsc	STATUS, Z
	bra		TMR_Stop

	movlw	0x01				;  CMD_TMR
	subwf	CMD_TMR+0, F
	clrw
	subwfb	CMD_TMR+1, F
	return

TMR_Stop:
	bcf		T2CON, TMR2ON		;  CMD_TMR ,   TMR2
	return

;-------------------------------------------------------------------------
;    W  (CMD_TMR = W * 100)
TMR_Set_:
	movwf	TmpREG+0			; TmpREG = W
	clrf	TmpREG+1
	clrf	CMD_TMR+0
	clrf	CMD_TMR+1
	call	REG_Shift			; TmpREG=*2
	call	REG_Shift			; TmpREG=*2
	call	REG_Add				; CMD_TMR = TmpREG*4
	call	REG_Shift			; TmpREG=*2
	call	REG_Shift			; TmpREG=*2
	call	REG_Shift			; TmpREG=*2
	call	REG_Add				; CMD_TMR = TmpREG*4 + TmpREG*32
	call	REG_Shift			; TmpREG=*2
	bra		REG_Add				; CMD_TMR = TmpREG*4 + TmpREG*32 + Tmp*64 = Tmp*(4+32+64)

REG_Shift:
	lslf	TmpREG+0, F
	rlf		TmpREG+1, F
	return

REG_Add:
	movf	TmpREG+0, W
	addwf	CMD_TMR+0, F
	movf	TmpREG+1, W
	addwfc	CMD_TMR+1, F
	return

;-------------------------------------------------------------------------
;  State
State_Set_:
	bcf		State, 0
	bcf		State, 1
	bcf		State, 2
	iorwf	State, F
	return

;=========================================================================
;     UART
RCEvent:
	btfsc	State, 7			;   
	bra		UART_OverRD			; 

	movf	UART_WR, W			;   
	movwf	FSR0L
	clrf	FSR0H
	movlb	3
	movf	RCREG, W			;    
	movlb	0
	movwf	INDF0
	incf	UART_WR, F			;    

	movlw	0x70				;    , ..
	xorwf	UART_WR, W
	movlw	UART_Buf			;    
	btfsc	STATUS, Z
	movwf	UART_WR
	
	movf	UART_RD, W			;    
	xorwf	UART_WR, W
	bcf		State, 7
	btfsc	STATUS, Z
	bsf		State, 7			;    
	return

UART_OverRD:	
	movlb	3
	movf	RCREG, W			;      RCIF
	movlb	0
	return

;-------------------------------------------------------------------------
;       (   )
UART_Recv:
	clrf	TMR2				;   CMD_TMR
	bsf		T2CON, TMR2ON
	bcf		State, 5
	movf	UART_RD, W			;   UART_RD
	movwf	FSR0L
	clrf	FSR0H

UART_Recv_Loop:
	btfsc	State, 7			;   , ..
	bra		UART_Recv_Done		;   
	movf	UART_RD, W			; ,  
	xorwf	UART_WR, W
	btfss	STATUS, Z			;   ,  ..
	bra		UART_Recv_Done		;   
	btfsc	T2CON, TMR2ON		;    , ..
	bra		UART_Recv_Loop		; 
	bsf		State, 5			;  

UART_Recv_Done:
	bcf		T2CON, TMR2ON		;    CMD_TMR
	movf	INDF0, W			;     
	return

;-------------------------------------------------------------------------
;      
UART_Next:
	incf	UART_RD, F			;     1 
	movlw	0x70				;     
	xorwf	UART_RD, W
	movlw	UART_Buf			;    
	btfsc	STATUS, Z			;   
	movwf	UART_RD
	bcf		State, 7
	return

;-------------------------------------------------------------------------
;     
UART_Count:
	movf	UART_RD, W			
	subwf	UART_WR, W			; UART_WR - UART_RD
	btfsc	STATUS, Z
	bra		UART_CheckOver		;   0 -   
	btfss	STATUS, C			; ,  
	addlw	D'80'
	return
UART_CheckOver:
	btfsc	State, 7
	addlw	D'80'
	return

;=========================================================================
;   EEPROM    /
EEPROM_SetAddr:
	movlb	3
	movwf	EEADRL				;   
	clrf	EEADRH
	movlb	0
	return

;----------------------------------------------------------------------
;    EEPROM
EEPROM_Read:
	movlb	3
	bcf		EECON1, CFGS
	bcf		EECON1, EEPGD		;    EEPROM
	bsf		EECON1, RD			; 
	incf	EEADRL, F			;  
	movf	EEDATL, W			;    EEPROM
	movlb	0
	return

;-------------------------------------------------------------------------
;    EEPROM    W 
EEPROM_Read_
	call	EEPROM_SetAddr
	bra		EEPROM_Read

;----------------------------------------------------------------------
;    EEPROM
EEPROM_Write:
	movlb	3
	movwf	EEDATL
	bcf		EECON1, CFGS
	bcf		EECON1, EEPGD		;  EEPROM
	call	EEPROM_Magic
	incf	EEADRL, F			;  
	bsf		INTCON, GIE			;  
	movlb	0
	return

EEPROM_Magic:
	bsf		EECON1, WREN		;     Flash
	bcf		INTCON, GIE			;   
	movlw	0x55				;  
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf		EECON1, WR			; 
	btfsc	EECON1, WR			;   
	bra		$-1
	bcf		EECON1, WREN		;     Flash
	bsf		INTCON, GIE			;  
	return

;=========================================================================
;   c   
UART_SendASCII:
	movlb	3
	movwf	TXREG				;     
	btfss	TXSTA, TRMT			;   
	bra		$-1
	movlb	0
	return

;----------------------------------------------------------------------
;    EEPROM
EE_Send:
	call	EEPROM_SetAddr		;   EEPROM  
	call	EEPROM_Read			;  
EE_LEN:
	movwf	CMD_LEN				;    
EE_Next:
	call	EEPROM_Read			;   
	call	UART_SendASCII		;   UART
	decfsz	CMD_LEN, F			;     ..
	bra		EE_Next				;  -
	return

;----------------------------------------------------------------------
;  UTF-
UART_SendUTF:
	call	EEPROM_SetAddr		;   
	movlw	0x00				; 0x00
	call	UART_SendASCII		;   1=  
	call	EEPROM_Read			;  1-    ( )
	call	UART_SendASCII		;   2- 
	bra		EE_LEN				;     

;----------------------------------------------------------------------
;   (FSR1 -  )
UART_Send:
	movwf	FSR1L				;     
	movlw	HIGH CMD | 0x80		;     
	movwf	FSR1H				;   
	moviw	FSR1++				;   =  
	movwf	CMD_LEN

UART_Send_Next:
	moviw	FSR1++				;     UART
	call	UART_SendASCII
	decfsz	CMD_LEN, F
	bra		UART_Send_Next
	return

;----------------------------------------------------------------------
;  
UART_Query:
	movwf	CMD_ADR				;   
	movlw	'A'					;   "AT"  UART
	call	UART_SendASCII
	movlw	'T'
	call	UART_SendASCII
	movf	CMD_ADR, W			;   ..
	bra		UART_Send			;  

;----------------------------------------------------------------------
;    
UART_QueryCR:
	call	UART_Query

;----------------------------------------------------------------------
;     (0x0D)
UART_SendCR:
	movlw	0x0D
	bra		UART_SendASCII

;----------------------------------------------------------------------
;    CMD_ADR 
UART_Wait:
	movwf	CMD_ADR				;   ,    
UART_Wait_:
	movf	CMD_ADR, W			;    
	movwf	FSR1L
	movlw	HIGH RPL | 0x80
	movwf	FSR1H
	moviw	FSR1++				;   
	movwf	CMD_LEN
	bcf		State, 6			;     

UART_WaitNext:
	call	UART_Recv			;    
	RPL_Timeout	MAIN_Ret		;    - 
	moviw	FSR1++				;     
	xorwf	INDF0, W			;     
	btfsc	STATUS, Z
	bra		UART_WaitNext_		;   -   

	btfss	State, 6			;    ?
	call	UART_Next			;  -  
	bra		UART_Wait_			;  -   ,   

UART_WaitNext_:
	call	UART_Next			;  
	bsf		State, 6			;     
	decfsz	CMD_LEN, F			;   ?
	bra		UART_WaitNext		;  -  
	return						;  - 

;=========================================================================
;  CMD_LEN  DivVAL (10)
Div10:
	movlw	D'160'
	movwf	DivVAL
	clrf	RetVAL
	call	Div10_
	call	Div10_
	call	Div10_
	call	Div10_
	call	Div10_
	movf	CMD_LEN, W
	return

Div10_:
	subwf	CMD_LEN, F
	rlf		RetVAL, F
	btfss	RetVAL, 0
	addwf	CMD_LEN, F
	lsrf	DivVAL, F
	movf	DivVAL, W
	return

;----------------------------------------------------------------------
;       (CMD_LEN, DivVAL, RetVAL, VAL_Buf)
UART_SendVAL:
	movwf	CMD_LEN
	call	Div10				; 
	movwf	VAL_Buf+2

	movf	RetVAL, W
	movwf	CMD_LEN
	call	Div10				; 
	movwf	VAL_Buf+1

	movf	RetVAL, W
	movwf	CMD_LEN
	call	Div10				; 
	movwf	VAL_Buf+0

	bcf		State, 6			;    

	btfss	STATUS, Z			;   ,  ..
	call	UART_SendVAL_DIG	;  
	movf	VAL_Buf+1, W
	btfss	State, 6
	btfss	STATUS, Z			;    - ..
	call	UART_SendVAL_DIG	;  
	movf	VAL_Buf+2, W		;  - ..
	bra		UART_SendVAL_DIG	;    

UART_SendVAL_DIG:
	bsf		State, 6			;    
	iorlw	0x30
	bra		UART_SendASCII

;----------------------------------------------------------------------
;      (RetVAL -  )
UART_RecvVAL:
	clrf	RetVAL

UART_RecvVAL_Next:
	call	UART_Recv			;    
	sublw	0x2F				; 0x2F - Byte
	btfsc	STATUS, C
	return						; Byte < 0x30
	sublw	0xFF
	sublw	0x09				; 9 - Byte
	btfsc	STATUS, C			;   ,  ..
	bra		UART_AddVAL			;   RetVAL = RetVAL * 10  + 
	return

UART_AddVAL:
	lslf	RetVAL, W			; W = RetVAL * 2
	movwf	DivVAL				; DivVAL = RetVAL * 2
	lslf	DivVAL, F			; DivVAL = RetVAL * 4
	lslf	DivVAL, F			; DivVAL = RetVAL * 8
	addwf	DivVAL, F			; DivVAL = RetVAL * 8 + RetVAL * 2 = RetVAL * 10
	movf	INDF0, W
	andlw	0x0F
	addwf	DivVAL, W			; W = RetVAL * 10 + 
	movwf	RetVAL
	call	UART_Next			;   
	bra		UART_RecvVAL_Next

;-------------------------------
;  UART  SIM800
UART_SelectSIM800:
	movlb	2					;  UART  SIM800L
	bcf		APFCON, RXDTSEL
	bcf		APFCON, TXCKSEL
	movlb	0
	movlw	LOW SIM_Speed		;     SIM
	bra		UART_SetSpeed

;-------------------------------
;  UART  RS485
UART_SelectRS485:
	movlb	2					;  UART  RS485
	bsf		APFCON, RXDTSEL
	bsf		APFCON, TXCKSEL
	movlb	0
	movlw	LOW RS485_Speed		;     SIM

UART_SetSpeed:
	call	EEPROM_Read_		;  EEPROM
	andlw	0x7
	addwf	WREG, F				; 2
	addlw	LOW UART_SpeedTable	; +SpeedTable
	movwf	FSR1L				;     
	movlw	HIGH UART_SpeedTable | 0x80	;     
	movwf	FSR1H				;   

	movlb	3					;    SPBRGL
	moviw	FSR1++
	movwf	SPBRGL
	moviw	FSR1++
	movwf	SPBRGH
	movlb	0
	return

UART_SpeedTable:
	dt	LOW UART_1200, HIGH UART_1200
	dt	LOW UART_2400, HIGH UART_2400
	dt	LOW UART_4800, HIGH UART_4800
	dt	LOW UART_9600, HIGH UART_9600
	dt	LOW UART_19200, HIGH UART_19200
	dt	LOW UART_38400, HIGH UART_38400
	dt	LOW UART_57600, HIGH UART_57600
	dt	LOW UART_115200, HIGH UART_115200

;-------------------------------------------------------------------------
;   SMS
MAIN_SMS:
	CMD_QueryCR	CMD_CMGL		;   SMS-
	TMR_Set D'2'				; 2 
	RPL_Wait RPL_CMGL			;  "+CMGL: "
	RPL_Timeout	MAIN_Ret

	TMR_Set	D'20'				;       20sec  
	RPL_Wait RPL_Number			;  "+7"

	call	RS485_Reset			;    RS485
	movlw	D'10'				; 12    SMS
	movwf	CMD_LEN

MAIN_SMS_Number:
	call	UART_Recv
	movwi	FSR1++
	call	UART_Next
	decfsz	CMD_LEN, F
	bra	MAIN_SMS_Number

	RPL_Wait RPL_OK				;  OK
	RPL_Timeout	MAIN_Ret
	
	CMD_Query CMD_CMGS			;  SMS
	movlw	D'10'				;  
	movwf	CMD_LEN
	call	RS485_Reset

MAIN_SMS_Number_Send:
	moviw	FSR1++
	call	UART_SendASCII
	decfsz	CMD_LEN, F
	bra	MAIN_SMS_Number_Send
	movlw	'"'					;     
	movwi	FSR1++
	call	UART_SendASCII
	call	UART_SendCR

	RPL_Wait RPL_Input			;  ">"
	RPL_Timeout	MAIN_Ret

	;    SMS  

	movlw	0x07
	andwf	State, W	
	call	UART_SendVAL
	movlw	0x1A
	call	UART_SendASCII

	RPL_Wait RPL_CMGS
	RPL_Wait RPL_OK				;  OK
	
	CMD_QueryCR	CMD_CMGD		;   SMS-
	TMR_Set D'10'				; 2 s
	RPL_Wait RPL_OK				;  OK

MAIN_Ret:
	return

;-------------------------------------------------------------------------
;  
MAIN_Reset:	
	reset

;-------------------------------------------------------------------------
;   State
StateSet_:
	bcf		State, 0
	bcf		State, 1
	bcf		State, 2
	iorwf	State, F
	return
;-------------------------------------------------------------------------
;     RS485
RS485_Reset:
	movlw	RS485_Buf
	movwf	FSR1L
	clrf	FSR1H
	return
;=========================================================================
;    JP1
JP_Cycle:
	btfss	PORTA, 1
	bcf		LATA, 4
	btfsc	PORTA, 1
	bsf		LATA, 4
	btfss	PORTA, 5
	bcf		LATA, 0
	btfsc	PORTA, 5
	bsf		LATA, 0
	bra		JP_Cycle

;=========================================================================
; 
MAIN_Start:
	movlb	2
	bsf		LATA, 4
	bcf		LATA, 2

	movlb	1
	movlw	0x70				;    32MHz
	movwf	OSCCON
	bcf		OPTION_REG, TMR0CS	;  TMR0  Fosc/4/256 = 31250 (32us)

	bcf		TRISA, 2			;  Reset SIM800
	bcf		TRISA, 0			; TX- SIM800
	bcf		TRISA, 4			; TX- RS-485

	movlb	3
	clrf	ANSELA

	movlb	0
	btfss	PORTA, 3			;   ,  ..
	bra		JP_Cycle			;    RS-485  SIM800L

	movlb	1
	bsf		PIE1, TMR2IE		;    TMR2
	bsf		PIE1, RCIE			;    UART

	movlb	3
	bsf		BAUDCON, BRG16		;   UART  c 16- 
	bsf		TXSTA, BRGH			;    
	bsf		RCSTA, SPEN			;  RX  TX 
	bsf		RCSTA, CREN
	bsf		TXSTA, TXEN
	movlb	0
	call	UART_SelectSIM800	;   SIM800

	movlw	D'125'				; Fosc/4/64/125/10 = 100Hz (0.01s) -   TMR2
	movwf	PR2
	movlw	0x4B
	movwf	T2CON

	bsf		INTCON, PEIE		;    
	bsf		INTCON, GIE       	;   

	clrf	State				;  
	clrf	CMD_TMR+0			;  
	clrf	CMD_TMR+1
	movlw	UART_Buf			;     
	movwf	UART_WR
	movwf	UART_RD

;=========================================================================
;  
MAIN_Handle:
;-------------------------------
;   SIM800
	movlb	2
	bsf		LATA, 2				;   SIM800
	movlb	0

	TMR_Set	D'2'
	clrf	TMR2
	bsf		T2CON, TMR2ON
	btfsc	T2CON, TMR2ON
	bra		$-1

	movlb	2
	bcf		LATA, 2				;   SIM800
	movlb	0

;-------------------------------	
	TMR_Set	D'10'				; 10 sec
	RPL_Wait RPL_SMS_Ready		;  "SMS Ready"
	RPL_Timeout	MAIN_Reset		;  
;-------------------------------
MAIN_Setup:						;  SIM-
;-------------------------------
	call	MAIN_SMS			;   SMS

	CMD_QueryCR CMD_CIPSHUT		; => AT+CIPSHUT<CR> -   IP-
	TMR_Set D'10'
	RPL_Wait RPL_SHUT_OK		;  SHUT OK<CR>
	RPL_Timeout	MAIN_Reset

;-------------------------------
	CMD_Query CMD_CIPRXGET		; => AT+CIPRXGET= -    
	CMD_ASCII '1'				; => 1
	call	UART_SendCR			; => <CR>
	TMR_Set	D'10'
	RPL_Wait RPL_OK				;  OK<CR>
	RPL_Timeout	MAIN_Setup

;-------------------------------
	StateSet	1				;      Internet
;-------------------------------
	CMD_Query CMD_CSTT			; => AT+CSTT<APN><CR> -  APN
	movlw	LOW APN
	call	EE_Send
	call	UART_SendCR

	TMR_Set	D'20'
	RPL_Wait RPL_OK				;  OK<CR>
	RPL_Timeout	MAIN_Setup	

;-------------------------------
	CMD_QueryCR CMD_CIICR		; => AT+CIICR<CR> -   APN
	TMR_Set	D'20'
	RPL_Wait RPL_OK				;  OK<CR>
	RPL_Timeout	MAIN_Setup

;-------------------------------
	CMD_QueryCR CMD_CIFSR		; => AT+CIFSR<CR> -   IP-
	TMR_Set	D'10'
	RPL_Wait RPL_CR				;  <CR>   IP-
	RPL_Timeout	MAIN_Setup

;-------------------------------
	StateSet	2				;      MQTT
;-------------------------------
	CMD_Query CMD_CIPSTART		; => AT+CIPSTART -   MQTT-
	movlw	LOW MQTT_Server		; => "<>","<>"
	call	EE_Send
	call	UART_SendCR			; => <CR>
	TMR_Set	D'20'
	RPL_Wait RPL_CONNECT_OK		;  CONNECT OK<CR>
	RPL_Timeout	MAIN_Setup

;-------------------------------
	StateSet	3				;     CONNECT
;-------------------------------
	;   MQTT- (CONNECT)
	movlw	D'16'				;  2*3      + 10   
	movwf	MQTT_Size			
	movlw	LOW MQTT_ID			; +
	call	EEPROM_Read_		;   MQTT_ID
	addwf	MQTT_Size, F
	movlw	LOW MQTT_UserName	; +
	call	EEPROM_Read_		;   MQTT_UserName
	addwf	MQTT_Size, F
	movlw	LOW MQTT_Password	; +
	call	EEPROM_Read_		;   MQTT_Password
	addwf	MQTT_Size, F
	
	;   SIM-  
	call	MQTT_Send

	;    
	CMD_ASCII 0x10				; => MQTT_Type = CONNECT
	movf	MQTT_Size, W		; => MQTT_Size
	call	UART_SendASCII

	movlw	LOW MQTT_CONNECT	; =>  MQTT_CONNECT
	call	UART_Send

	movlw	LOW MQTT_ID			; => MQTT_ID
	call	UART_SendUTF

	movlw	LOW MQTT_UserName	; => MQTT_UserName
	call	UART_SendUTF

	movlw	LOW MQTT_Password	; => MQTT_Password
	call	UART_SendUTF

;-------------------------------
	call	MQTT_RecvAnswer		;   MQTT-

	TMR_Set	D'10'
	RPL_Wait MQTT_CONNACK		;  MQTT_CONNACK
	RPL_Timeout	MAIN_Setup		;  
	RPL_Wait RPL_OK				;  "OK"<CR>
	RPL_Timeout	MAIN_Setup		;  

;-------------------------------
	StateSet	4				;     
;-------------------------------
	;   MQTT- (SUBSCRIBE)
	movlw	D'5'				; 2 (PacketID) + 2 (Topic) + 1 (QoS)
	movwf	MQTT_Size			
	movlw	LOW MQTT_Request	; +
	call	EEPROM_Read_		;   MQTT_Request
	addwf	MQTT_Size, F
	
	;    
	call	MQTT_Send

	;    
	CMD_ASCII 0x82				; => MQTT_Type = SUBSRIBE
	movf	MQTT_Size, W		; => MQTT_Size
	call	UART_SendASCII

	movlw	LOW MQTT_SUBSCRIBE	; =>  MQTT_SUBSCRIBE
	call	UART_Send

	movlw	LOW MQTT_Request	; => MQTT_Request
	call	UART_SendUTF

	CMD_ASCII 0x00				; => QoS0
	
;-------------------------------
	call	MQTT_RecvAnswer		;   MQTT-

	TMR_Set	D'10'
	RPL_Wait MQTT_SUBACK		;  MQTT_SUBACK
	RPL_Timeout	MAIN_Setup		;  
	RPL_Wait RPL_OK				;  "OK"<CR>
	RPL_Timeout	MAIN_Setup		;  

;-------------------------------
	StateSet	0				;     
;-------------------------------
MAIN_Cycle:						;  
;-------------------------------
	call	UART_SelectSIM800	

	call	MQTT_RecvMQTT		;  MQTT-
	RPL_Timeout	MAIN_Ping

	decf	MQTT_Size, F
	call	UART_Recv			;   MQTT-
	movwf	CMD_ADR
	xorlw	0x30				;    PUBLISH-
	btfss	STATUS, Z
	bra		MAIN_Flush			; ,   TCP-

	decf	MQTT_Size, F
	call	UART_Next			;  
	call	UART_Recv			;     MQTT-
	andlw	0x80				;  7- 
	btfss	STATUS, Z			;   ,       
	bra		MAIN_Flush			;   TCP-
	
	movf	INDF0, W			;   NQTT-  CMD_LEN (..     TCP-)
	movwf	CMD_LEN
	call	UART_Next			;  
	movlw	LOW MQTT_Request
	call	EEPROM_Read_		;   MQTT_Request
	addlw	D'2'				; +2   
	movwf	RetVAL
	subwf	CMD_LEN, F			;     =  

MAIN_SkipTopicName:
	call	UART_Recv			;   
	call	UART_Next
	decfsz	RetVAL, F
	bra		MAIN_SkipTopicName

MAIN_DataWait:					;   
	call	UART_Count			;     
	movwf	RS485_LEN
	movf	CMD_LEN, W
	subwf	RS485_LEN, W		; LEN - CMD_LEN
	btfss	STATUS, C			;       
	bra		MAIN_DataWait		;  -    

	call	UART_SelectRS485

	clrf	FSR0H				;    RS485
	movf	RS485_LEN, W
	movwf	CMD_LEN

MAIN_RS485Send:
	movf	UART_RD, W			;   UART_RD
	movwf	FSR0L
	movf	INDF0, W			;  
	call	UART_SendASCII		;   RS-485
	call	UART_Next
	decfsz	CMD_LEN, F
	bra		MAIN_RS485Send

	movlw	LOW RS485_Timeout	;    
	call	EEPROM_Read_
	movwf	CMD_TMR+0
	clrf	CMD_TMR+1
	clrf	TMR2
	bsf		T2CON, TMR2ON		;   CMD_TMR

	btfsc	T2CON, TMR2ON
	bra		$-1	

	call	UART_Count			;    
	movwf	RS485_LEN
	btfsc	STATUS, Z
	bra		MAIN_Cycle			;    -    

	call	RS485_Reset			;     RS485
	movf	RS485_LEN, W
	movwf	CMD_LEN

MAIN_RS485Recv:					;     
	movf	UART_RD, W			;   UART_RD
	movwf	FSR0L
	movf	INDF0, W			;  
	movwi	FSR1++				;   
	call	UART_Next
	decfsz	CMD_LEN, F
	bra		MAIN_RS485Recv

	call	UART_SelectSIM800
	
	;   MQTT- (PUBLISH)
	movlw	D'2'				; 2 (Topic)
	movwf	MQTT_Size			
	movlw	LOW MQTT_Response	; +
	call	EEPROM_Read_		;   MQTT_Response
	addwf	MQTT_Size, F		; +
	movf	RS485_LEN, W		;  
	addwf	MQTT_Size, F
	
	;    
	call	MQTT_Send

	;    
	CMD_ASCII 0x30				; => MQTT_Type = PUBLISH
	movf	MQTT_Size, W		; => MQTT_Size
	call	UART_SendASCII

	movlw	LOW MQTT_Response	; => MQTT_Response
	call	UART_SendUTF

	call	RS485_Reset			; =>   
	movf	RS485_LEN, W
	movwf	CMD_LEN
	
MAIN_MQTTSend:
	moviw	FSR1++
	call	UART_SendASCII		;   RS-485
	decfsz	CMD_LEN, F
	bra		MAIN_MQTTSend

	TMR_Set D'10'				;   10s
	RPL_Wait RPL_SEND_OK		;  "SEND OK"<CR>
	RPL_Timeout MAIN_Setup
	bra 	MAIN_Cycle			;    

MAIN_Flush:						;    TCP-
	call	UART_Recv			
	call	UART_Next
	decfsz	MQTT_Size, F
	bra		MAIN_Flush
	bra		MAIN_Cycle			;   

;-------------------------------------------------------------------------
MAIN_Ping:						;  PINGREQ
	clrf	MQTT_Size			;   2- MQTT-
	call	MQTT_Send
	CMD_ASCII 0xC0				;  MQTT_PINGREQ
	CMD_ASCII 0x00
	call	MQTT_RecvAnswer		;  
	RPL_Timeout	MAIN_Setup
	TMR_Set	D'2'
	RPL_Wait MQTT_PINGRESP		;  MQTT_PINGRESP
	RPL_Wait RPL_OK				;  "OK"<CR>
	RPL_Timeout	MAIN_Setup

	call	MAIN_SMS			;   SMS
	bra		MAIN_Cycle

;-------------------------------------------------------------------------
;  SIM-   MQTT-
MQTT_Send:
	CMD_Query CMD_CIPSEND		; => AT+CIPSEND= 
	movlw	 D'2'				; => MQTT_Size + 1 (MQTT_Type) + 1 (MQTT_Size)
	addwf	 MQTT_Size, W
	call	 UART_SendVAL
	call	 UART_SendCR		; => <CR>

	TMR_Set  D'10'				;   10s
	RPL_Wait RPL_Input			;  ">"
	return

;-------------------------------------------------------------------------
;   MQTT- 
MQTT_RecvAnswer:
	TMR_Set	D'10'
	RPL_Wait RPL_SEND_OK		;  "SEND OK"<CR>
	RPL_Timeout	MAIN_Ret

MQTT_RecvMQTT:
	CMD_Query CMD_CIPRXGET		; => AT+CIPRXGET=
	CMD_ASCII '4'				; => 4 -    
	call	UART_SendCR			; => <CR>
	TMR_Set	D'10'
	RPL_Wait RPL_RECV			;  "+CIPRXGET: "
	call	 UART_RecvVAL		;   4
	RPL_Timeout	MAIN_Ret
	
	decf	RetVAL, W			;   "+CIPRXGET: 1",  ..
	btfsc	STATUS, Z
	bra		MQTT_RecvWait_		;  
	
	call	 UART_Next			; ,  
	call	 UART_RecvVAL		;     RetVAL
	RPL_Wait RPL_CR				;  <CR>
	RPL_Timeout	MAIN_Ret

	movf	RetVAL, W			;  TCP-
	movwf	MQTT_Size			;   MQTT_Size
	btfsc	STATUS, Z
	bra		MQTT_RecvWait		;   - 

	CMD_Query CMD_CIPRXGET		; => AT+CIPRXGET= -   TCP/IP ()
	CMD_ASCII '2'				; => 2,
	CMD_ASCII ','
	movf	RetVAL, W			; =>  
	call	UART_SendVAL
	call	UART_SendCR			; => <CR>

	TMR_Set  D'10'				;   10s
	RPL_Wait RPL_RECV			;  "+CIPRXGET: "
	call	 UART_RecvVAL		;  1- 
	call	 UART_Next			;  
	call	 UART_RecvVAL		;  2- 
	call	 UART_Next			;  
	call	 UART_RecvVAL		;  3-  -  
	RPL_Wait RPL_CR				;  <CR>
	return

;-------------------------------------------------------------------------
MQTT_RecvWait:
	TMR_Set	D'20'
	RPL_Wait RPL_RECV			;  "+CIPRXGET: "
	call	UART_RecvVAL		;    RetVAL (  1)
MQTT_RecvWait_:
	RPL_Wait RPL_CR				;  <CR>
	RPL_Timeout	MAIN_Ret
	bra		MQTT_RecvMQTT		;     MQTT-

;-------------------------------------------------------------------------
; 
CMD	ORG 0x0600

CMD_CIPSHUT
	dt	CMD_CSTT-CMD_CIPSHUT-1, "+CIPSHUT"
CMD_CSTT
	dt	CMD_CIICR-CMD_CSTT-1, "+CSTT="
CMD_CIICR
	dt	CMD_CIFSR-CMD_CIICR-1, "+CIICR"
CMD_CIFSR
	dt	CMD_CIPSTART-CMD_CIFSR-1, "+CIFSR"
CMD_CIPSTART
	dt 	CMD_CIPSEND-CMD_CIPSTART-1, "+CIPSTART=\"TCP\","
CMD_CIPSEND
	dt	CMD_CIPRXGET-CMD_CIPSEND-1, "+CIPSEND="
CMD_CIPRXGET
	dt	CMD_CMGL-CMD_CIPRXGET-1, "+CIPRXGET="
CMD_CMGL
	dt	CMD_CMGD-CMD_CMGL-1, "+CMGL=\"ALL\""
CMD_CMGD
	dt	CMD_CMGS-CMD_CMGD-1, "+CMGD=1,1"
CMD_CMGS
	dt	CMD_End-CMD_CMGS-1, "+CMGS=\"+7"
CMD_End

;-------------------------------------------------------------------------
; MQTT-
MQTT_CONNECT
	dt	MQTT_SUBSCRIBE-MQTT_CONNECT-1, 0x00, 0x04, "MQTT", 0x04, B'11000010', 0x00, D'120'
MQTT_SUBSCRIBE
	dt	MQTT_EndRequest-MQTT_SUBSCRIBE-1, 0x00, 0x01
MQTT_EndRequest

;-------------------------------------------------------------------------
; 
RPL	ORG 0x0700

RPL_CR
	dt	RPL_Input-RPL_CR-1, 0x0D, 0x0A
RPL_Input
	dt	RPL_SMS_Ready-RPL_Input-1, 0x0D, 0x0A, ">"
RPL_SMS_Ready
	dt	RPL_OK-RPL_SMS_Ready-1, 0x0D, 0x0A, "SMS Ready", 0x0D, 0x0A
RPL_OK
	dt	RPL_SHUT_OK-RPL_OK-1, "0", 0x0D, 0x0A
RPL_SHUT_OK
	dt	RPL_CONNECT_OK-RPL_SHUT_OK-1, 0x0D, 0x0A, "SHUT OK", 0x0D, 0x0A
RPL_CONNECT_OK
	dt	RPL_SEND_OK-RPL_CONNECT_OK-1, 0x0D, 0x0A, "CONNECT OK", 0x0D, 0x0A
RPL_SEND_OK
	dt	RPL_RECV-RPL_SEND_OK-1, 0x0D, 0x0A, "SEND OK", 0x0D, 0x0A
RPL_RECV
	dt	RPL_CMGL-RPL_RECV-1, 0x0D, 0x0A, "+CIPRXGET: "
RPL_CMGL
	dt	RPL_Number-RPL_CMGL-1, "+CMGL: "
RPL_Number
	dt	RPL_CMGS-RPL_Number-1, "+7"
RPL_CMGS
	dt	RPL_End-RPL_CMGS-1, "+CMGS:"
RPL_End

;-------------------------------------------------------------------------
; MQTT-
MQTT_CONNACK
	dt	MQTT_SUBACK-MQTT_CONNACK-1, 0x20, 0x02, 0x00, 0x00
MQTT_SUBACK
	dt	MQTT_PINGRESP-MQTT_SUBACK-1, 0x90, 0x03, 0x00, 0x01, 0x00
MQTT_PINGRESP
	dt	MQTT_EndResponse-MQTT_PINGRESP-1, 0xD0, 0x00
MQTT_EndResponse

;-------------------------------------------------------------------------
; EEPROM
DATAEE	ORG	0xF000

SIM_Speed						;    SIM800L
	DE 3						;  3   UART_SpeedTable (9600)
RS485_Speed						;    RS485
	DE 3						;  3   UART_SpeedTable (9600)
RS485_Timeout
	DE D'5'						;     RS485 = 50ms
APN
	DE MQTT_Server-APN-1, "\"internet.mts.ru\""
MQTT_Server
	DE MQTT_ID-MQTT_Server-1, "\"srv2.clusterfly.ru\",\"9991\""
MQTT_ID
	DE MQTT_UserName-MQTT_ID-1, "ModbusTool"
MQTT_UserName
	DE MQTT_Password-MQTT_UserName-1, "login"
MQTT_Password
	DE MQTT_Request-MQTT_Password-1, "password"
MQTT_Request
	DE MQTT_Response-MQTT_Request-1, "request"
MQTT_Response
	DE DATA_EE_End-MQTT_Response-1, "response"
DATA_EE_End
    END